<?php
/**
 * DBSessionHandler.inc.php
 * Code by Strangecode :: www.strangecode.com :: This document contains copyrighted information.
 * @author  Quinn Comendant <quinn@strangecode.com>
 * @version 1.3
 * @since   1999
 */

class DBSessionHandler {

    var $db; // DB object.

    var $_params = array(
        'db_server' => 'localhost',
        'db_name' => '',
        'db_user' => '',
        'db_pass' => '',
        'db_table' => 'session_tbl',
        'create_table' => true, // Automatically create table and verify columns. Better set to false after site launch.
    );

    /**
     * Constructor
     *
     * @access  public
     * @param
     * @return
     * @author  Quinn Comendant <quinn@strangecode.com>
     * @since   18 Jul 2005 11:02:50
     */
    function DBSessionHandler($db=null, $params=array())
    {
        $this->_params = array_merge($this->_params, $params);

        if (isset($db)) {
            if (is_a($db, 'DB')) {
                if ($db->isConnected()) {
                    // Use existing db connection.
                    $this->db =& $db;
                } else {
                    App::logMsg(sprintf('Provided DB object is not connected. %s', mysql_error($db->dbh)), LOG_ERR, __FILE__, __LINE__);
                }
            } else {
                App::logMsg(sprintf('Provided DB object is not valid. %s', gettype($db)), LOG_ERR, __FILE__, __LINE__);
            }
        } else {
            // Create our own new db connection.
            require_once dirname(__FILE__) . '/DB.inc.php';

            $this->db =& new DB();
            $this->db->setParam(array(
                'db_server' => $this->_params['db_server'],
                'db_name' => $this->_params['db_name'],
                'db_user' => $this->_params['db_user'],
                'db_pass' => $this->_params['db_pass'],
                'db_always_debug' => $this->_params['db_always_debug'],
                'db_debug' => $this->_params['db_debug'],
                'db_die_on_failure' => $this->_params['db_die_on_failure'],
            ));

            // Connect to database.
            $this->db->connect();
        }

        if (!isset($this) || !is_a($this->db, 'DB') || !$this->db->isConnected()) {
            trigger_error('Invalid DB object or unable to connect to database.', E_USER_ERROR);
        }

        // Get create tables config from global context.
        if (!is_null(App::getParam('db_create_tables'))) {
            $this->_params['create_table'] = App::getParam('db_create_tables');
        }

        // Ensure db table is fit.
        $this->initDB();

        ini_set('session.save_handler', 'user');
        session_set_save_handler(
            array(&$this, 'dbSessionOpen'),
            array(&$this, 'dbSessionClose'),
            array(&$this, 'dbSessionRead'),
            array(&$this, 'dbSessionWrite'),
            array(&$this, 'dbSessionDestroy'),
            array(&$this, 'dbSessionGarbage')
        );
    }

    /**
     * Setup the database table for this class.
     *
     * @access  public
     * @author  Quinn Comendant <quinn@strangecode.com>
     * @since   26 Aug 2005 17:09:36
     */
    function initDB($recreate_db=false)
    {
        static $_db_tested = false;

        if ($recreate_db || !$_db_tested && $this->_params['create_table']) {
            if ($recreate_db) {
                $this->db->query("DROP TABLE IF EXISTS " . $this->_params['db_table']);
                App::logMsg(sprintf('Dropping and recreating table %s.', $this->_params['db_table']), LOG_DEBUG, __FILE__, __LINE__);
            }
            $this->db->query("CREATE TABLE IF NOT EXISTS " . $this->_params['db_table'] . " (
                session_id char(32) NOT NULL default '',
                session_data mediumtext NOT NULL,
                last_access timestamp(14) NOT NULL,
                PRIMARY KEY (session_id),
                KEY last_access (last_access)
            )");

            if (!$this->db->columnExists($this->_params['db_table'], array('session_id', 'session_data', 'last_access'))) {
                App::logMsg(sprintf('Database table %s has invalid columns. Please update this table manually.', $this->_params['db_table']), LOG_ALERT, __FILE__, __LINE__);
                trigger_error(sprintf('Database table %s has invalid columns. Please update this table manually.', $this->_params['db_table']), E_USER_ERROR);
            }
        }
        $_db_tested = true;
    }

    function dbSessionOpen($save_path, $sess_name)
    {
        return true;
    }

    function dbSessionClose()
    {
        return true;
    }

    function dbSessionRead($session_id)
    {
        // Select the data belonging to session $session_id from the session table
        $qid = $this->db->query("SELECT session_data FROM " . $this->_params['db_table'] . " WHERE session_id = '" . $this->db->escapeString($session_id) . "'");

        // Return the session data that was found
        if (mysql_num_rows($qid) == 1) {
            $row = mysql_fetch_row($qid);
            return $row[0];
        }

        // NOTICE: Output is expected to be an empty string always rather than 'false'.
        return '';
    }

    function dbSessionWrite($session_id, $session_data)
    {
        // Write the serialized session data ($session_data) to the session table
        $this->db->query("REPLACE INTO " . $this->_params['db_table'] . "(session_id, session_data, last_access) VALUES ('" . $this->db->escapeString($session_id) . "', '" . $this->db->escapeString($session_data) . "', null)");

        return true;
    }

    function dbSessionDestroy($session_id)
    {
        // Delete from the table all data for the session $session_id
        $this->db->query("DELETE FROM " . $this->_params['db_table'] . " WHERE session_id = '" . $this->db->escapeString($session_id) . "'");

        return true;
    }

    function dbSessionGarbage($max_lifetime=4000)
    {
        // Delete old values from the session table
        $qid = $this->db->query("DELETE FROM " . $this->_params['db_table'] . " WHERE UNIX_TIMESTAMP(last_access) < " . (time() - $max_lifetime));

        return true;
    }
}

?>
